@@ -15,12 +15,16 @@
     This module is managing the Copy and Paste functions.
 """
 
+import traceback
+import logging
+from itertools import chain
+import PySide
+
 import ogAST
 import sdlSymbols
 import genericSymbols
-import logging
 import Renderer
-import PySide
+import Pr
 
 __all__ = ['copy', 'paste']
 
@@ -32,6 +36,7 @@ COPY_PASTE = []
 # Actual scene clipboard
 CLIPBOARD = None
 
+
 def copy(selection):
     ''' Create a copy (duplicate) of the selected symbols in AST form '''
     # Clear the copy paste buffer
@@ -60,13 +65,13 @@ def copy(selection):
 def copy_branch(top_level_item):
     ''' Copy branches (recursively) '''
     res_terminators = []
-    item_ast, terminators = top_level_item.get_ast()
+    pr_text = '\n'.join(Pr.generate(top_level_item, cpy=True,
+                                    nextstate=False, recursive=True))
+    item_ast, terminators = top_level_item.get_ast(pr_text)
     LOG.debug('COPY ' + str(item_ast))
-
-    # Set absolute (scene) coordinates of top level item
-    scene_pos = top_level_item.scenePos()
-    item_ast.abs_x = scene_pos.x()
-    item_ast.abs_y = scene_pos.y()
+    if not item_ast:
+        LOG.error('ERROR - copy failed')
+        return
 
     branch = [item_ast]
 
@@ -97,6 +102,7 @@ def copy_branch(top_level_item):
                 res_terminators.extend(term_inators)
     return branch, res_terminators
 
+
 def paste(parent, scene):
     '''
         Paste previously copied symbols at selection point
@@ -120,33 +126,43 @@ def paste_floating_objects(scene):
         states = [i for i in item_list if isinstance(i, ogAST.State)]
         text_areas = (i for i in item_list if isinstance(i, ogAST.TextArea))
         labels = (i for i in item_list if isinstance(i, ogAST.Floating_label))
+        procedures = (i for i in item_list if isinstance(i, ogAST.Procedure))
+        processes = (i for i in item_list if isinstance(i, ogAST.Process))
         for state in states:
             # First check if state has already been pasted
             try:
                 new_item = Renderer.render(state, scene=CLIPBOARD,
                            terminators=terminators, states=states)
             except TypeError as err:
-                LOG.debug('No paste "'+ state.inputString + '" -' + str(err))
+                LOG.debug('No paste "' + state.inputString + '" -' + str(err))
                 # Discard terminators (explanation given in Renderer._state)
                 pass
             else:
                 LOG.debug('PASTE STATE "' + state.inputString + '"')
                 symbols.append(new_item)
                 # Insert the new state at click coordinates
-                scene.addItem(new_item)
-        for text_area in text_areas:
-            LOG.debug('PASTE TEXT AREA')
-            new_item = Renderer.render(text_area, scene)
+                Renderer.add_to_scene(new_item, scene)
+        for each in chain(text_areas, labels):
+            LOG.debug('PASTE Text Area/Label')
+            new_item = Renderer.render(each, scene, states=states)
             symbols.append(new_item)
-        for label in labels:
-            LOG.debug('PASTE LABEL')
-            new_item = Renderer.render(label, scene, states=states)
+        for each in chain(procedures, processes):
+            LOG.debug('PASTE Process/Procedure')
+            new_item = Renderer.render(each, scene, states=states)
             symbols.append(new_item)
+            new_item.nested_scene = scene.create_subscene(
+                                               type(new_item).__name__.lower())
+            # Render recursively, creating any required scene
+            try:
+                new_item.nested_scene.render_everything(each.content)
+            except TypeError as err:
+                LOG.debug(str(err))
+
         if start:
             start, = start
             LOG.debug('PASTE START')
-            for item in scene.items():
-                if isinstance(item, sdlSymbols.Start) and item.isVisible():
+            for item in scene.visible_symb:
+                if isinstance(item, sdlSymbols.Start):
                     raise TypeError('Only one START symbol is possible')
             new_item = Renderer.render(start, scene, states=states)
             symbols.append(new_item)
@@ -155,7 +171,7 @@ def paste_floating_objects(scene):
 
 def paste_below_item(parent, scene):
     ''' Paste items under a selected symbol '''
-    LOG.debug('Pasting below item ' + repr(parent)[slice(0,20)])
+    LOG.debug('Pasting below item ' + repr(parent)[slice(0, 20)])
     symbols = []
     for item_list, _ in COPY_PASTE:
         states = [i for i in item_list if isinstance(i, ogAST.State)]
@@ -168,7 +184,7 @@ def paste_below_item(parent, scene):
             # Check that item is compatible with parent
             if (type(new_item).__name__ in parent.allowed_followers):
                 # Move the item from the clipboard to the scene
-                scene.addItem(new_item)
+                Renderer.add_to_scene(new_item, scene)
                 new_item.setPos(0, 0)
                 symbols.append(new_item)
             else:
