@@ -15,18 +15,21 @@
     Contact: maxime.perrotin@esa.int
 """
 
-import os
-import sys
 import math
 import logging
-from collections import namedtuple
 
-from PySide.QtCore import Qt, QPointF, QLineF
+from PySide.QtCore import Qt, QPointF, QLineF, Slot
 
 from PySide.QtGui import QGraphicsPathItem, QPainterPath, QGraphicsItem, QPen,\
                          QPainter, QFont, QGraphicsTextItem, QColor, \
                          QFontMetrics
 
+import ogParser
+from TextInteraction import EditableText
+
+LOG = logging.getLogger(__name__)
+
+
 # pylint: disable=R0904
 class Connection(QGraphicsPathItem, object):
     ''' Connection between two symbols (top-level class) '''
@@ -39,7 +42,7 @@ class Connection(QGraphicsPathItem, object):
         super(Connection, self).__init__(parent)
         self.parent = parent
         self.child = child
-        self._start_point =  None
+        self._start_point = None
         self._end_point = None
         self._middle_points = []
         pen = QPen()
@@ -91,14 +94,13 @@ class Connection(QGraphicsPathItem, object):
             angle = math.pi * 2 - angle
         arrow_size = 10.0
         arrow_p1 = end_point + QPointF(
-                math.sin(angle - math.pi/3) * arrow_size,
-                math.cos(angle - math.pi/3) * arrow_size)
+                math.sin(angle - math.pi / 3) * arrow_size,
+                math.cos(angle - math.pi / 3) * arrow_size)
         arrow_p2 = end_point + QPointF(
-                math.sin(angle - math.pi + math.pi/3) * arrow_size,
-                math.cos(angle - math.pi + math.pi/3) * arrow_size)
+                math.sin(angle - math.pi + math.pi / 3) * arrow_size,
+                math.cos(angle - math.pi + math.pi / 3) * arrow_size)
         return (arrow_p1, arrow_p2)
 
-
     def draw_arrow_head(self, shape, origin='head', kind='simple'):
         ''' Generic function to draw a simple arrow '''
         if kind == 'simple':
@@ -235,13 +237,75 @@ class CommentConnection(Connection):
         yield QPointF(go_to_point, self.end_point.y())
 
 
-class Channel(Connection):
+class SignalList(EditableText):
+    ''' Simplified text editor for signal lists '''
+    def __init__(self, parent, text='', hyperlink = None):
+        ''' Smaller font than normal text '''
+        super(SignalList, self).__init__(parent, text)
+        self.setFont(QFont('Ubuntu', pointSize=8))
+
+    def set_text_alignment(self):
+        ''' Text justification - ignore '''
+        pass
+
+    def set_textbox_position(self):
+        ''' Alignment vs parent - ignore, done by the parent '''
+        pass
+
+    def try_resize(self):
+        ''' Resizing of the parent when size expands - ignore '''
+        pass
+
+    # pylint: disable=C0103
+    def focusOutEvent(self, event):
+        ''' Redefined function - update in_sig and out_sig in parent '''
+        super(SignalList, self).focusOutEvent(event)
+
+
+class Signalroute(Connection):
     ''' Subclass of Connection used to draw channels between processes '''
+    in_sig = out_sig = None
     def __init__(self, process):
         ''' Set generic parameters from Connection class '''
-        super(Channel, self).__init__(process, process)
-        self.text_label = None
+        super(Signalroute, self).__init__(process, process)
+        self.parser = ogParser
+        self.completion_list = []
+        self.blackbold = ()
+        self.redbold = ()
+        self.label_in = SignalList(parent=self)
+        self.label_out = SignalList(parent=self)
+        if not Signalroute.in_sig:
+            # keep at class level as long as only one process is supported
+            # when copy-pasting a process the challel in/out signal lists
+            # are not parsed. Workaround is to keep the list "global"
+            # to allow a copy of both process and channel
+            # Needed for the image exporter, that copies the scene to a
+            # temporary one
+            Signalroute.in_sig = '{}'.format(',\n'.join(sig['name']
+                                   for sig in process.input_signals))
+            Signalroute.out_sig = '{}'.format(',\n'.join(sig['name']
+                                    for sig in process.output_signals))
+        self.label_in.setPlainText('[{}]'.format(self.in_sig))
+        self.label_out.setPlainText('[{}]'.format(self.out_sig))
+        self.label_in.document().contentsChanged.connect(self.change_siglist)
+        self.label_out.document().contentsChanged.connect(self.change_siglist)
+        for each in (self.label_in, self.label_out):
+            each.show()
         self.process = process
+        self.reshape()
+
+    @Slot()
+    def change_siglist(self):
+        ''' Called when user modified the text of label_in or label_out '''
+        for each in self.label_in, self.label_out:
+            if not each.hasFocus():
+                if not unicode(each).startswith('['):
+                    each.setPlainText('[{}'.format(unicode(each)))
+                if not unicode(each).endswith(']'):
+                    each.setPlainText('{}]'.format(unicode(each)))
+        Signalroute.in_sig = unicode(self.label_in)[1:-1]
+        Signalroute.out_sig = unicode(self.label_out)[1:-1]
+
 
     @property
     def start_point(self):
@@ -253,11 +317,45 @@ class Channel(Connection):
     def end_point(self):
         ''' Compute connection end point - redefined function '''
         # Arrow always bumps at the screen edge
-        view = self.scene().views()[0]
-        view_pos = view.mapToScene(
+        try:
+            view = self.scene().views()[0]
+            view_pos = view.mapToScene(
                            view.viewport().geometry()).boundingRect().topLeft()
-        scene_pos_x = self.mapFromScene(view_pos).x()
-        return QPointF(scene_pos_x, self.start_point.y())
+            scene_pos_x = self.mapFromScene(view_pos).x()
+            return QPointF(scene_pos_x, self.start_point.y())
+        except (IndexError, AttributeError):
+            # In case there is no view (e.g. Export PNG from cmd line)
+            return QPointF(self.start_point.x() - 250, self.start_point.y())
+
+    def reshape(self):
+        ''' Redefine shape function to add the text areas '''
+        super(Signalroute, self).reshape()
+
+        width_in = self.label_in.boundingRect().width()
+
+        self.label_in.setX(self.start_point.x() - width_in)
+        self.label_in.setY(self.start_point.y() + 5)
+        self.label_out.setX(self.end_point.x() + 10)
+        self.label_out.setY(self.end_point.y() + 5)
+
+    def check_syntax(self, text):
+        ''' Check the syntax of the IN and OUT signal lists '''
+        try:
+            _, syntax_errors, _, _, _ = self.parser.parseSingleElement(
+                                           'signalroute', text)
+        except (AssertionError, AttributeError) as err:
+            LOG.error('Checker failed:' + str(err))
+        else:
+            return syntax_errors
+
+    def update_completion_list(self, pr_text):
+        ''' Called after text has been edited '''
+        # TODO - call parseSingleElement, check sdlSymbols for examples
+        pass
+
+    def resize_item(self, new_rect):
+        ''' Called after signallist text has been edited '''
+        pass
 
 
 class Controlpoint(QGraphicsPathItem, object):
@@ -301,7 +399,6 @@ class Connectionpoint(Controlpoint):
         # Symbol actually owning the connection point
         self.symbol = symbol
 
-
     def update_position(self):
         '''
             Update the position of the end point so that it is always
