@@ -11,6 +11,8 @@
         inner_labels_to_floating(process) : remove labels from transitions
         map_input_state(process) -> mapping: create a mapping
                                     input-state-transition
+        sorted_fields(SEQ/CHOICE) : returns the ordered list of fields
+                                    of an ASN.1 SEQUENCE or CHOICE type
 
     Copyright (c) 2012-2014 European Space Agency
 
@@ -19,6 +21,7 @@
     Contact: maxime.perrotin@esa.int
 """
 
+import operator
 import logging
 from itertools import chain
 from collections import defaultdict
@@ -30,7 +33,7 @@ import ogAST
 LOG = logging.getLogger(__name__)
 
 __all__ = ['flatten', 'rename_everything', 'inner_labels_to_floating',
-           'map_input_state']
+           'map_input_state', 'sorted_fields']
 
 
 def map_input_state(process):
@@ -71,8 +74,8 @@ def inner_labels_to_floating(process):
             process.content.floating_labels.append(new_floating)
 
 
-def flatten(process, sep='_'):
-    ''' Flatten the nested states:
+def flatten(process, sep=u'_'):
+    ''' In-place update of the AST: flatten a model with nested states
         Rename inner states, procedures, etc. and move them to process level
     '''
     def update_terminator(context, term, process):
@@ -80,19 +83,19 @@ def flatten(process, sep='_'):
         if term.inputString.lower() in (st.statename.lower()
                                 for st in context.composite_states):
             if not term.via:
-                term.next_id = term.inputString.lower() + sep + 'START'
+                term.next_id = term.inputString.lower() + sep + u'START'
             else:
-                term.next_id = '{term}{sep}{entry}{sep}START'.format(
+                term.next_id = u'{term}{sep}{entry}_START'.format(
                         term=term.inputString, entry=term.entrypoint, sep=sep)
         elif term.inputString.strip() == '-':
             term.candidate_id = defaultdict(list)
             for each in term.possible_states:
-                if each in (st.statename.lower()
+                if each.lower() in (st.statename.lower()
                             for st in context.composite_states):
-                    term.candidate_id[each + sep + 'START'] = \
+                    term.candidate_id[each + sep + u'START'] = \
                                        [st for st in process.mapping.viewkeys()
                                         if st.startswith(each)
-                                        and not st.endswith(sep + 'START')]
+                                        and not st.endswith(u'START')]
                 else:
                     term.candidate_id[-1].append(each)
 
@@ -105,7 +108,7 @@ def flatten(process, sep='_'):
         set_terminator_states(state, prefix)
         set_transition_states(state, prefix)
 
-        state.mapping = {prefix + key:state.mapping.pop(key)
+        state.mapping = {prefix + key: state.mapping.pop(key)
                          for key in state.mapping.keys()}
         process.transitions.extend(state.transitions)
 
@@ -146,7 +149,7 @@ def flatten(process, sep='_'):
                          if isinstance(trans, int)):
                 call_entry = ogAST.ProcedureCall()
                 call_entry.inputString = 'entry'
-                call_entry.output = [{'outputName': prefix + 'entry',
+                call_entry.output = [{'outputName': prefix + u'entry',
                                      'params': [], 'tmpVars': []}]
                 process.transitions[each].actions.insert(0, call_entry)
 
@@ -157,7 +160,7 @@ def flatten(process, sep='_'):
                 if each.terminator.kind == 'return':
                     call_exit = ogAST.ProcedureCall()
                     call_exit.inputString = 'exit'
-                    call_exit.output = [{'outputName': prefix + 'exit',
+                    call_exit.output = [{'outputName': prefix + u'exit',
                                      'params': [], 'tmpVars': []}]
                     each.actions.append(call_exit)
 
@@ -252,6 +255,7 @@ def rename_everything(ast, from_name, to_name):
     '''
     _, _, _ = ast, from_name, to_name
 
+
 @rename_everything.register(ogAST.Automaton)
 def _rename_automaton(ast, from_name, to_name):
     ''' Renaming at Automaton top level (content of digragrams) '''
@@ -297,6 +301,7 @@ def _rename_label(ast, from_name, to_name):
         ast.inputString = to_name
     rename_everything(ast.transition, from_name, to_name)
 
+
 @rename_everything.register(ogAST.Decision)
 def _rename_decision(ast, from_name, to_name):
     ''' Rename elements in decision '''
@@ -312,7 +317,7 @@ def _rename_answer(ast, from_name, to_name):
     if ast.kind in ('constant', 'open_range'):
         rename_everything(ast.constant, from_name, to_name)
     elif ast.kind == 'closed_range':
-        pass # TODO when supported
+        pass  # TODO when supported
     rename_everything(ast.transition, from_name, to_name)
 
 
@@ -323,6 +328,7 @@ def _rename_transition(ast, from_name, to_name):
         # Label, output, task, decision, terminators
         rename_everything(each, from_name, to_name)
 
+
 @rename_everything.register(ogAST.Terminator)
 def _rename_terminator(ast, from_name, to_name):
     ''' Rename terminators: join/labels, next_state '''
@@ -338,6 +344,7 @@ def _rename_task_assign(ast, from_name, to_name):
     for each in ast.elems:
         rename_everything(each, from_name, to_name)
 
+
 @rename_everything.register(ogAST.TaskForLoop)
 def _rename_forloop(ast, from_name, to_name):
     ''' List of FOR loops '''
@@ -347,6 +354,7 @@ def _rename_forloop(ast, from_name, to_name):
         rename_everything(each['range']['stop'], from_name, to_name)
         rename_everything(each['transition'], from_name, to_name)
 
+
 @rename_everything.register(ogAST.ExprPlus)
 @rename_everything.register(ogAST.ExprMul)
 @rename_everything.register(ogAST.ExprMinus)
@@ -371,19 +379,21 @@ def _rename_expr(ast, from_name, to_name):
     rename_everything(ast.left, from_name, to_name)
     rename_everything(ast.right, from_name, to_name)
 
-@rename_everything.register(ogAST.PrimPath)
+
 @rename_everything.register(ogAST.PrimVariable)
 def _rename_path(ast, from_name, to_name):
     ''' Ultimate seek point for the renaming: primary path/variables '''
     if ast.value[0].lower() == from_name.lower():
         ast.value[0] = to_name
 
-@rename_everything.register(ogAST.PrimIfThenElse)
+
+@rename_everything.register(ogAST.PrimConditional)
 def _rename_ifhthenelse(ast, from_name, to_name):
-    ''' Rename expressions in If-Then-Else-Fi construct '''
+    ''' Rename expressions in Conditional expression construct '''
     for expr in ('if', 'then', 'else'):
         rename_everything(ast.value[expr], from_name, to_name)
 
+
 def find_labels(trans):
     '''
         Yield a list of transition actions whenever a label is found
@@ -404,7 +414,8 @@ def find_labels(trans):
             new_trans = ogAST.Transition()
             # Create a floating label
             flab = ogAST.Floating_label(label=action)
-            new_trans.actions = trans.actions[slice(idx+1, len(trans.actions))]
+            new_trans.actions = \
+                    trans.actions[slice(idx + 1, len(trans.actions))]
             new_trans.terminator = trans.terminator
             new_trans.terminators = trans.terminators
             flab.transition = new_trans
@@ -423,8 +434,16 @@ def find_labels(trans):
                 for new_fl in find_labels(answer.transition):
                     # Append the remaining actions of the transition
                     if not new_fl.transition.terminator:
-                        new_fl.transition.actions.extend(
-                               trans.actions[slice(idx+1, len(trans.actions))])
+                        new_fl.transition.actions.extend(trans.actions
+                                          [slice(idx + 1, len(trans.actions))])
                         new_fl.transition.terminator = trans.terminator
                     yield new_fl
 
+
+def sorted_fields(atype):
+    ''' Return the sorted list of a SEQUENCE or CHOICE type fields '''
+    if atype.kind not in ('SequenceType', 'ChoiceType'):
+        raise TypeError('Not a SEQUENCE nor a CHOICE')
+    tmp = ([k, val.Line, val.CharPositionInLine]
+             for k, val in atype.Children.viewitems())
+    return (x[0] for x in sorted(tmp, key=operator.itemgetter(1,2)))
