@@ -34,11 +34,22 @@ import ogAST
 import sdlSymbols
 import genericSymbols
 import logging
+from itertools import chain
 from singledispatch import singledispatch
 
 LOG = logging.getLogger(__name__)
 
-__all__ = ['render']
+__all__ = ['render', 'add_to_scene']
+
+
+def add_to_scene(item, scene):
+    ''' Add item to a scene after verifying that the scene allows it '''
+    if type(item) in scene.allowed_symbols:
+        scene.addItem(item)
+    else:
+        #print type(item), scene.allowed_symbols, scene.context
+        raise TypeError('This symbol does not fit the current scene')
+
 
 @singledispatch
 def render(ast, scene, parent, states, terminators=None):
@@ -50,27 +61,39 @@ def render(ast, scene, parent, states, terminators=None):
     raise TypeError('[Renderer] Unsupported symbol in branch: ' + repr(ast))
 
 
+@render.register(ogAST.Block)
+def _block(ast, scene):
+    ''' Render a block, containing a set of process symbols '''
+    # TODO = Add text areas with signal lists, external procedures defs...
+    top_level = []
+    for each in ast.processes:
+        top_level.append(render(each, scene))
+    return top_level
+
+
 @render.register(ogAST.Process)
-def _process(ast, scene):
-    ''' Render the symbols inside a process or procedure '''
+def _process(ast, scene, **_):
+    ''' Render a Process symbol (in a BLOCK diagram) '''
     # Set autocompletion lists for input, output, state, types, variables:
-    try:
-        sdlSymbols.TextSymbol.completion_list = {
-                t.replace('-', '_') for t in ast.dataview}
-    except (AttributeError, TypeError):
-        LOG.debug('No dataview for filling types autocompletion list')
-    sdlSymbols.State.completion_list = {
-            state for state in ast.mapping if state != 'START'}
-    sdlSymbols.Input.completion_list = {
-            signal['name'] for signal in ast.input_signals}
-    sdlSymbols.Output.completion_list = {
-            signal['name'] for signal in ast.output_signals}
-    sdlSymbols.Task.completion_list = set(ast.variables.keys())
+#   try:
+#       sdlSymbols.TextSymbol.completion_list = {
+#               t.replace('-', '_') for t in ast.dataview}
+#   except (AttributeError, TypeError):
+#       LOG.debug('No dataview for filling types autocompletion list')
+#   sdlSymbols.State.completion_list = {
+#           state for state in ast.mapping if state != 'START'}
+#   sdlSymbols.Input.completion_list = {
+#           signal['name'] for signal in ast.input_signals}
+#   sdlSymbols.Output.completion_list = {
+#           signal['name'] for signal in ast.output_signals}
+    #sdlSymbols.Task.completion_list = set(ast.variables.keys())
+#
+#   sdlSymbols.ProcedureCall.completion_list = {
+#           proc.inputString for proc in ast.procedures}
 
-    sdlSymbols.ProcedureCall.completion_list = {
-            proc.inputString for proc in ast.procedures}
-
-    return render(ast.content, scene)
+    symbol = sdlSymbols.Process(ast, ast)
+    add_to_scene(symbol, scene)
+    return symbol
 
 
 @render.register(ogAST.Automaton)
@@ -91,16 +114,26 @@ def _automaton(ast, scene):
     if ast.start:
         top_level_symbols.append(render(ast.start, scene, ast.states))
 
+    # Render named start symbols in nested states
+    for each in ast.named_start:
+        top_level_symbols.append(render(each, scene, ast.states))
+
     # Render floating labels
     for label in ast.floating_labels:
         top_level_symbols.append(render(label, scene, ast.states))
 
     # Render floating states
+    nested_states = []
     for state in ast.states:
         # Create only floating states
         try:
             new_state = render(state, scene=scene, states=ast.states,
                                terminators=ast.parent.terminators)
+            if new_state.nested_scene:
+                if unicode(new_state).lower() in nested_states:
+                    new_state.nested_scene = None
+                else:
+                    nested_states.append(unicode(new_state).lower())
         except TypeError:
             # Discard terminators (see _state function for explanation)
             pass
@@ -124,10 +157,13 @@ def _state(ast, scene, states, terminators, parent=None):
             raise TypeError('This state is a terminator')
     new_state = sdlSymbols.State(parent=None, ast=ast)
     if new_state not in scene.items():
-        scene.addItem(new_state)
+        add_to_scene(new_state, scene)
+
+    for exit in chain(ast.inputs, ast.connects):
+        render(exit, scene=scene, parent=new_state, states=states)
+
+    new_state.nested_scene = ast.composite or ogAST.CompositeState()
 
-    for inp in ast.inputs:
-        render(inp, scene=scene, parent=new_state, states=states)
     return new_state
 
 
@@ -136,7 +172,7 @@ def _procedure(ast, scene, parent=None, states=None):
     ''' Add a procedure symbol to the scene '''
     _, _ = parent, states
     proc_symbol = sdlSymbols.Procedure(ast, ast)
-    scene.addItem(proc_symbol)
+    add_to_scene(proc_symbol, scene)
     return proc_symbol
 
 
@@ -145,7 +181,7 @@ def _text_area(ast, scene, parent=None, states=None):
     ''' Render a text area from the AST '''
     _, _ = parent, states
     text = sdlSymbols.TextSymbol(ast)
-    scene.addItem(text)
+    add_to_scene(text, scene)
     return text
 
 
@@ -154,7 +190,18 @@ def _start(ast, scene, states, parent=None):
     ''' Add the start symbol to a scene '''
     _ = parent
     start_symbol = sdlSymbols.Start(ast)
-    scene.addItem(start_symbol)
+    add_to_scene(start_symbol, scene)
+    if ast.transition:
+        render(ast.transition, scene=scene, parent=start_symbol, states=states)
+    return start_symbol
+
+
+@render.register(ogAST.CompositeState_start)
+def _composite_start(ast, scene, states, parent=None):
+    ''' Add an editable start symbol to a scene (in composite states) '''
+    _ = parent
+    start_symbol = sdlSymbols.StateStart(ast)
+    add_to_scene(start_symbol, scene)
     if ast.transition:
         render(ast.transition,
                       scene=scene, parent=start_symbol, states=states)
@@ -166,10 +213,9 @@ def _procedure_start(ast, scene, states, parent=None):
     ''' Add the procedure start symbol to a scene '''
     _ = parent
     start_symbol = sdlSymbols.ProcedureStart(ast)
-    scene.addItem(start_symbol)
+    add_to_scene(start_symbol, scene)
     if ast.transition:
-        render(ast.transition,
-                      scene=scene, parent=start_symbol, states=states)
+        render(ast.transition, scene=scene, parent=start_symbol, states=states)
     return start_symbol
 
 
@@ -179,27 +225,22 @@ def _floating_label(ast, scene, states, parent=None):
     _ = parent
     lab = sdlSymbols.Label(parent=None, ast=ast)
     if lab not in scene.items():
-        scene.addItem(lab)
+        add_to_scene(lab, scene)
     lab.setPos(ast.pos_x, ast.pos_y)
     if ast.transition:
-        render(ast.transition,
-                      scene=scene,
-                      parent=lab,
-                      states=states)
+        render(ast.transition, scene=scene, parent=lab, states=states)
     return lab
 
 
 @render.register(ogAST.Transition)
 def _transition(ast, scene, parent, states):
     ''' Add a transition to a scene '''
-    for action_symbol in ast.actions:
+    for each in ast.actions:
         # pylint: disable=E1111
-        parent = render(action_symbol,
-                               scene=scene, parent=parent, states=states)
+        parent = render(each, scene=scene, parent=parent, states=states)
 
     if ast.terminator:
-        render(ast.terminator,
-                      scene=scene, parent=parent, states=states)
+        render(ast.terminator, scene=scene, parent=parent, states=states)
 
 
 @render.register(ogAST.Comment)
@@ -224,7 +265,7 @@ def _output(ast, scene, parent, states):
 
 
 @render.register(ogAST.ProcedureCall)
-def _output(ast, scene, parent, states):
+def _procedure_call(ast, scene, parent, states):
     ''' Create an OUTPUT or PROCEDURE CALL symbol '''
     _, _ = scene, states
     return sdlSymbols.ProcedureCall(parent, ast=ast)
@@ -238,8 +279,8 @@ def _decision(ast, scene, parent, states):
     if not parent:
         symbol.setPos(ast.pos_x, ast.pos_y)
     for branch in ast.answers:
-        render(branch,
-                      scene=scene, parent=symbol, states=states)
+        render(branch, scene=scene, parent=symbol, states=states)
+    symbol.updateConnectionPointPosition()
     return symbol
 
 
@@ -259,8 +300,7 @@ def _answer(ast, scene, parent, states):
     if not parent:
         symbol.setPos(ast.pos_x, ast.pos_y)
     if ast.transition:
-        render(ast.transition,
-                      scene=scene, parent=symbol, states=states)
+        render(ast.transition, scene=scene, parent=symbol, states=states)
     return symbol
 
 
@@ -269,8 +309,7 @@ def _terminator(ast, scene, parent, states):
     ''' Create a TERMINATOR symbol '''
     if ast.label:
         # pylint: disable=E1111
-        parent = render(ast.label,
-                               scene=scene, parent=parent, states=states)
+        parent = render(ast.label, scene=scene, parent=parent, states=states)
     if ast.kind == 'next_state':
         LOG.debug('ADDING NEXT_STATE ' + ast.inputString)
         # Create a new state symbol
@@ -282,12 +321,15 @@ def _terminator(ast, scene, parent, states):
                     state_ast.pos_x == ast.pos_x and
                     state_ast.pos_y == ast.pos_y):
                 LOG.debug('MERGING TERMINATOR "' + ast.inputString + '"')
-                for input_ast in state_ast.inputs:
-                    render(input_ast,
-                                  scene=scene, parent=symbol, states=states)
+                symbol.nested_scene = state_ast.composite or \
+                                      ogAST.CompositeState()
+                for each in chain(state_ast.inputs, state_ast.connects):
+                    render(each, scene=scene, parent=symbol, states=states)
+                break
+        symbol.nested_scene = ast.composite or ogAST.CompositeState()
     elif ast.kind == 'join':
         symbol = sdlSymbols.Join(parent, ast)
-    elif ast.kind == 'stop':
+    elif ast.kind in ('return', 'stop'):
         symbol = sdlSymbols.ProcedureStop(parent, ast)
     else:
         raise TypeError('Unsupported terminator: ' + repr(ast))
@@ -300,7 +342,7 @@ def _input(ast, scene, parent, states):
     # Note: PROVIDED clause is not supported
     inp = sdlSymbols.Input(parent, ast=ast)
     if inp not in scene.items():
-        scene.addItem(inp)
+        add_to_scene(inp, scene)
     if not parent:
         inp.setPos(ast.pos_x, ast.pos_y)
     if ast.transition:
@@ -309,3 +351,19 @@ def _input(ast, scene, parent, states):
                parent=inp,
                states=states)
     return inp
+
+
+@render.register(ogAST.Connect)
+def _connect(ast, scene, parent, states):
+    ''' Add connect symbol from the AST to the scene '''
+    conn = sdlSymbols.Connect(parent, ast=ast)
+    if conn not in scene.items():
+        add_to_scene(conn, scene)
+    if not parent:
+        conn.setPos(ast.pos_x, ast.pos_y)
+    if ast.transition:
+        render(ast.transition,
+               scene=scene,
+               parent=conn,
+               states=states)
+    return conn
