@@ -41,22 +41,23 @@ LOG = logging.getLogger(__name__)
 class Expression(object):
     ''' AST Entry for expressions - Always use subtype '''
     is_raw = False
+
     def __init__(self, inputString='', line=-1, charPositionInLine=-1):
         ''' Initialize Expression attributes '''
         self.inputString = inputString
         self.line = line
         self.charPositionInLine = charPositionInLine
-        # left and right are of type Expression (absent for primaries)
-        self.left = None
-        self.right = None
+        # Binary expressions have two sides
+        self.right = self.left = None
+
         # exprType is an ASN.1 type (as exported by asn1scc)
         self.exprType = None
         # Hint for code generators: intermediate storage identifier
         self.tmpVar = -1
 
-    def __repr__(self):
+    def trace(self):
         ''' Debug output for an expression '''
-        return '{exp} ({l},{c})'.format(exp=self.inputString,
+        return u'{exp} ({l},{c})'.format(exp=self.inputString,
                                         l=self.line,
                                         c=self.charPositionInLine)
 
@@ -75,14 +76,19 @@ class ExprMinus(Expression):
 
 class ExprOr(Expression):
     operand = 'or'
+    # Indicate that short circuit form is used ("or else")
+    shortcircuit = ''
 
 
 class ExprAnd(Expression):
     operand = 'and'
+    # Indicate that short circuit form is used ("and then")
+    shortcircuit = ''
 
 
 class ExprXor(Expression):
     operand = 'xor'
+    shortcircuit = ''
 
 
 class ExprEq(Expression):
@@ -118,16 +124,32 @@ class ExprMod(Expression):
 
 
 class ExprRem(Expression):
-    operand = 'mod'
+    operand = 'rem'
+
+
+class ExprNot(Expression):
+    operand = 'not'
+
+
+class ExprNeg(Expression):
+    operand = '-'
 
 
 class ExprAssign(Expression):
     operand = ':='
 
 
-class ExprAppend(Expression): pass
-class ExprIn(Expression): pass
-class ExprImplies(Expression): pass
+class ExprAppend(Expression):
+    pass
+
+
+class ExprIn(Expression):
+    pass
+
+
+class ExprImplies(Expression):
+    pass
+
 
 class Primary(Expression):
     '''
@@ -135,6 +157,7 @@ class Primary(Expression):
         Abstract class, never used directly, see subtypes below
     '''
     is_raw = True
+
     def __init__(self, inputString='', line=-1, charPositionInLine=-1,
                  primary=None):
         ''' Initialize common primary attributes '''
@@ -142,48 +165,76 @@ class Primary(Expression):
             self.inputString = primary.inputString
             self.line = primary.line
             self.charPositionInLine = primary.charPositionInLine
-            self.op_not = primary.op_not
-            self.op_minus = primary.op_minus
             self.value = primary.value
             self.exprType = primary.exprType
         else:
             self.inputString = inputString
             self.line = line
             self.charPositionInLine = charPositionInLine
-            self.op_not = False
-            self.op_minus = False
             self.value = None
             self.exprType = None
 
-    def __repr__(self):
+    def trace(self):
         ''' Debug output for a primary '''
-        return 'PRIMARY : {exp} ({l},{c})'.format(exp=self.inputString,
+        return u'PRIMARY : {exp} ({l},{c})'.format(exp=self.inputString,
                 l=self.line, c=self.charPositionInLine)
 
-# Subclasses of Primary - never use Primary directly
-class PrimPath(Primary):
-    ''' PrimPath is a list of elements needed to identify a value
-        For example, "i!j!k(5)" is stored as:
-        [ 'i', 'j', 'k', {'index':[Expression list]}]
-           (in that case, 5 is an index)
-        other example: "hello(world)" ->
-             ['hello', {'procParams':[Expression list]'}]
-        (in that case, hello is an operator and world is its parameter)
-    '''
+
+class PrimCall(Primary):
     is_raw = False
 
-class PrimVariable(PrimPath): pass # XXX should not be raw for codegen
-class PrimFPAR(PrimVariable): pass
-class PrimEnumeratedValue(Primary): pass
-class PrimInteger(Primary): pass
-class PrimReal(Primary): pass
-class PrimBoolean(Primary): pass
-class PrimConstant(Primary): is_raw = False #pass
-class PrimBitStringLiteral(Primary): pass   # Not supported yet
-class PrimOctetStringLiteral(Primary): pass # Not supported yet
+
+class PrimIndex(Primary):
+    is_raw = False
 
 
-class PrimIfThenElse(Primary):
+class PrimSubstring(Primary):
+    is_raw = False
+
+
+class PrimSelector(Primary):
+    is_raw = False
+
+
+class PrimVariable(Primary):
+    is_raw = False
+
+
+class PrimFPAR(PrimVariable):
+    pass
+
+
+class PrimEnumeratedValue(Primary):
+    pass
+
+
+class PrimInteger(Primary):
+    pass
+
+
+class PrimReal(Primary):
+    pass
+
+
+class PrimBoolean(Primary):
+    pass
+
+
+class PrimConstant(Primary):
+    is_raw = False
+
+
+class PrimBitStringLiteral(Primary):
+    ''' Not supported yet '''
+    pass
+
+
+class PrimOctetStringLiteral(Primary):
+    ''' Not supported yet '''
+    pass
+
+
+class PrimConditional(Primary):
     ''' value is a dictionnary:
         { 'if': Expression, 'then': Expression,
         'else': Expression, 'tmpVar': integer}
@@ -213,7 +264,8 @@ class PrimMantissaBaseExp(Primary):
     pass
 
 
-class PrimEmptyString(Primary): pass
+class PrimEmptyString(Primary):
+    pass
 
 
 class PrimChoiceItem(Primary):
@@ -221,7 +273,8 @@ class PrimChoiceItem(Primary):
     pass
 
 
-class PrimChoiceDeterminant(Primary): pass
+class PrimChoiceDeterminant(Primary):
+    pass
 
 
 class PrimSequenceOf(Primary):
@@ -259,9 +312,9 @@ class Decision(object):
         # hint for backends needing a temporary variable to hold the question
         self.tmpVar = -1
 
-    def __repr__(self):
+    def trace(self):
         ''' Debug output for a decision '''
-        return 'DECISION {exp} ({l},{c})'.format(exp=self.inputString,
+        return u'DECISION {exp} ({l},{c})'.format(exp=self.inputString,
                 l=self.line, c=self.charPositionInLine)
 
 
@@ -294,9 +347,9 @@ class Answer(object):
         # optional hyperlink
         self.hyperlink = None
 
-    def __repr__(self):
+    def trace(self):
         ''' Debug output for an answer '''
-        return 'ANSWER {exp} ({l},{c})'.format(exp=self.inputString,
+        return u'ANSWER {exp} ({l},{c})'.format(exp=self.inputString,
                 l=self.line, c=self.charPositionInLine)
 
 
@@ -317,9 +370,9 @@ class Task(object):
         self.hyperlink = None
         self.elems = []
 
-    def __repr__(self):
+    def trace(self):
         ''' Debug output for a task '''
-        return 'TASK {exp} ({l},{c})'.format(exp=self.inputString,
+        return u'TASK {exp} ({l},{c})'.format(exp=self.inputString,
                 l=self.line, c=self.charPositionInLine)
 
 
@@ -341,6 +394,7 @@ class TaskForLoop(Task):
     '''
     pass
 
+
 class Output(object):
     ''' AST Entry for OUTPUT statements '''
     def __init__(self, defName=''):
@@ -359,9 +413,9 @@ class Output(object):
         # optional hyperlink
         self.hyperlink = None
 
-    def __repr__(self):
+    def trace(self):
         ''' Debug output for an Output symbol '''
-        return '{exp} ({l},{c})'.format(exp=self.inputString,
+        return u'{exp} ({l},{c})'.format(exp=self.inputString,
                 l=self.line, c=self.charPositionInLine)
 
 
@@ -404,10 +458,12 @@ class Terminator(object):
         # There can be several if terminator follows a floating label
         # Note, this field is updated by the Helper.flatten function
         self.possible_states = []
+        # optional composite state content (type CompositeState)
+        self.composite = None
 
-    def __repr__(self):
+    def trace(self):
         ''' Debug output for terminators '''
-        return '{kind} {exp} ({l},{c}) at {x}, {y}'.format(
+        return u'{kind} {exp} ({l},{c}) at {x}, {y}'.format(
                 exp=self.inputString,
                 kind=self.kind.upper(), l=self.line, c=self.charPositionInLine,
                 x=self.pos_x, y=self.pos_y)
@@ -434,9 +490,9 @@ class Label(object):
         # Transition is used for floating labels (see AST entry)
         self.transition = None
 
-    def __repr__(self):
+    def trace(self):
         ''' Debug output for a label '''
-        return 'LABEL {label} ({l},{c})'.format(label=self.inputString,
+        return u'LABEL {label} ({l},{c})'.format(label=self.inputString,
                 l=self.line, c=self.charPositionInLine)
 
 
@@ -456,9 +512,9 @@ class Floating_label(Label, object):
             self.terminators = label.terminators
             self.transition = label.transition
 
-    def __repr__(self):
+    def trace(self):
         ''' Debug output for a label (used by code generator) '''
-        return 'CONNECTION {label} ({l},{c})'.format(label=self.inputString,
+        return u'CONNECTION {label} ({l},{c})'.format(label=self.inputString,
                 l=self.line, c=self.charPositionInLine)
 
 
@@ -479,12 +535,12 @@ class Transition(object):
         # procedure when the model is flattened.
         self.possible_states = []
 
-    def __repr__(self):
+    def trace(self):
         ''' Debug output: display all actions '''
-        data = [repr(action) for action in self.actions]
+        data = [action.trace() for action in self.actions]
         if self.terminator:
-            data.append(repr(self.terminator))
-        return '\n'.join(data)
+            data.append(self.terminator.trace())
+        return u'\n'.join(data)
 
 
 class Input(object):
@@ -517,9 +573,9 @@ class Input(object):
         # list of terminators following the input symbol
         self.terminators = []
 
-    def __repr__(self):
+    def trace(self):
         ''' Debug output for an INPUT symbol '''
-        return 'INPUT {exp} ({l},{c})'.format(exp=self.inputString,
+        return u'INPUT {exp} ({l},{c})'.format(exp=self.inputString,
                 l=self.line, c=self.charPositionInLine)
 
 
@@ -532,9 +588,9 @@ class Connect(Input):
         self.connect_list = []
         self.width = 1
 
-    def __repr__(self):
+    def trace(self):
         ''' Debug output for a CONNECT symbol '''
-        return 'CONNECT {exp} ({l},{c})'.format(exp=self.inputString,
+        return u'CONNECT {exp} ({l},{c})'.format(exp=self.inputString,
                 l=self.line, c=self.charPositionInLine)
 
 
@@ -556,9 +612,9 @@ class Start(object):
         # list of terminators following the start symbol
         self.terminators = []
 
-    def __repr__(self):
+    def trace(self):
         ''' Debug output for a START symbol '''
-        return 'START {}'.format(self.inputString)
+        return u'START {}'.format(self.inputString)
 
 
 class Procedure_start(Start):
@@ -586,9 +642,10 @@ class Comment(object):
         # optional hyperlink
         self.hyperlink = None
 
-    def __repr__(self):
+    def trace(self):
         ''' Debug output for a COMMENT symbol '''
-        return 'COMMENT {exp} ({l},{c})'.format(exp=self.inputString,
+        return u'COMMENT {exp} ({l},{c})'.format(
+                exp=self.inputString,
                 l=self.line, c=self.charPositionInLine)
 
 
@@ -617,9 +674,9 @@ class State(object):
         # optional composite state content (type CompositeState)
         self.composite = None
 
-    def __repr__(self):
+    def trace(self):
         ''' Debug output for a STATE symbol '''
-        return 'STATE {exp} ({l},{c}) at {x},{y}'.format(exp=self.inputString,
+        return u'STATE {exp} ({l},{c}) at {x},{y}'.format(exp=self.inputString,
                 l=self.line, c=self.charPositionInLine,
                 x=self.pos_x, y=self.pos_y)
 
@@ -630,8 +687,6 @@ class TextArea(object):
         ''' Text area (raw content for rendering only) '''
         self.inputString = '-- Declare your variables\n\n' \
                            '-- Syntax: DCL <variable name> <type name>;\n\n'
-        # DCL variables in the text area {name: (sort, default_value), ...}
-        self.variables = {}
         self.line = None
         self.charPositionInLine = None
         # Set default coordinates and width/height
@@ -639,12 +694,19 @@ class TextArea(object):
         self.pos_y = 0
         self.width = 170
         self.height = 140
+        # DCL variables in the text area {name: (sort, default_value), ...}
+        self.variables = {}
+        # fpar and timers are also listed, useful when using autocompletion
+        self.fpar = []
+        self.timers = []
+        # inner procedures - useful to get autocompletion params
+        self.procedures = []
         # optional hyperlink
         self.hyperlink = None
 
-    def __repr__(self):
+    def trace(self):
         ''' Debug output for a text area '''
-        return 'TEXTAREA {exp} ({l},{c})'.format(exp=self.inputString,
+        return u'TEXTAREA {exp} ({l},{c})'.format(exp=self.inputString,
                 l=self.line, c=self.charPositionInLine)
 
 
@@ -708,6 +770,9 @@ class Procedure(object):
         self.comment = None
         # Set of symbols contained in the procedure (type Automaton)
         self.content = Automaton(parent=self)
+        # input/output signal lists - unused but for context information
+        self.input_signals = []
+        self.output_signals = []
 
 
 class Process(object):
@@ -727,7 +792,8 @@ class Process(object):
         self.global_variables = {}
         self.global_timers = []
         # Set default coordinates and width/height
-        self.pos_x = self.pos_y = 150
+        self.pos_x = 250
+        self.pos_y = 150
         self.width = 150
         self.height = 75
         # Optional hyperlink
@@ -753,7 +819,7 @@ class Process(object):
 
         # list of operators (not supported) and procedures
         self.operators = {}
-        # list of Procedure (external procedures)
+        # list of Procedure (external and inner procedures)
         self.procedures = []
 
         # The Mapping structure should be used for code generation backends
@@ -794,9 +860,9 @@ class CompositeState(Process):
         # one nameless START, named START (one per entrypoint), states,
         # amd floating labels
 
-    def __repr__(self):
+    def trace(self):
         ''' Debug output for composite state '''
-        return 'COMPOSITE STATE {exp} ({l},{c})'.format(exp=self.statename,
+        return u'COMPOSITE STATE {exp} ({l},{c})'.format(exp=self.statename,
                 l=self.line, c=self.charPositionInLine)
 
 
@@ -843,7 +909,7 @@ class System(object):
         self.signals = []
         # list of ogAST.Procedure
         self.procedures = []
-        # channels: [{'name':str, 'routes':[{'source': str, 'dest': str, 
+        # channels: [{'name':str, 'routes':[{'source': str, 'dest': str,
         # 'signals': ['sig1', .. ]}]
         self.channels = []
         self.blocks = []
@@ -865,8 +931,9 @@ class AST(object):
         # Refs to the ASN.1 dataview AST (set with USE clauses)
         self.dataview = None
         self.asn1Modules = None
-        # Constants stored in the ASN.1 modules (type is unknown/unchecked)
-        self.asn1_constants = []
+        # ASN.1-defined constants (constants in Ada but variables in C)
+        # dictionnary: {ConstantName: type } - copied from dataview.py
+        self.asn1_constants = None
 
         # List of System
         self.systems = []
